@@ -72,14 +72,17 @@ def get_args() -> (str, str, argparse.Namespace):
         prov_parser.add_argument(
             "--image",
             metavar = "image",
-            #choices = provider_info[provider]['images'].keys(),
-            help="Specify the OS Image",
+            help="Specify the OS image",
         )
         prov_parser.add_argument(
             "--region",
             metavar = "region",
-            #choices = provider_info[provider]['regions'].keys(),
-            help="Specify the Region to Host the Image",
+            help="Specify the region to host the image",
+        )
+        prov_parser.add_argument(
+            "--resources",
+            metavar = "resources",
+            help="Specify the hardware resources for the host image",
         )
         prov_parser.add_argument("-s", "--setup", action="store_true", help="Create VPS'")
         prov_parser.add_argument("-l", "--list", action="store_true", help="List Available VPS'")
@@ -98,33 +101,25 @@ def get_args() -> (str, str, argparse.Namespace):
         return args
 
 
-    # Check for a default provider
-    if 'default' in cfg and cfg['default']:
-        default_provider = cfg['default']
-    else:
-        default_provider = None
-
-    # if default_provider is not chosen, update settings
-    if default_provider != args.provider:
-        default_provider = args.provider
-
     # check options for this provider
     for p in cfg['providers']:
-        if p['name'] == default_provider:
+        if p['name'] == args.provider:
 
-            if 'image' in p:
-                default_image = p['image']
-                if not args.image:
-                    args.image = default_image
+            if 'image' in p and not args.image:
+                    args.image = p['image']
 
-            if 'region' in p:
-                default_region = p['region']
-                if not args.region:
-                    args.region = default_region
+            # GCP only
+            if 'zone' in p and not args.zone:
+                args.zone = p['zone']
+
+            if 'region' in p and not args.region:
+                args.region = p['region']
+
+            if 'resources' in p and not args.resources:
+                args.resources = p['resources']
 
             if 'api_token' in p:
-                api_token = p['api_token']
-                args.api_token = api_token
+                args.api_token = p['api_token']
             break
     else:
         console.print(f"[red bold]{default_provider} is not a supported provider")
@@ -132,11 +127,22 @@ def get_args() -> (str, str, argparse.Namespace):
         parser.exit(0)
         sys.exit(1)
     
-    if args.provider == 'azure':
-        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(api_token, args.region)
+    provider_info[args.provider]['regions'] =  getattr(globals()[args.provider], 'get_regions')(args.api_token)
+
+    if args.provider == 'azure' or args.provider == 'aws':
+        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(args.api_token, args.region)
+        provider_info[args.provider]['resources'] =  getattr(globals()[args.provider], 'get_resources')(args.api_token, args.region)
+    elif args.provider == 'gcp':
+        # make sure the image/region combo is allowed 
+        valid_zone = getattr(globals()[args.provider], 'valid_zone')(args.api_token, args.zone)
+        assert valid_zone, f"{args.zone} is not valid"
+
+        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(args.api_token)
+        provider_info[args.provider]['resources'] =  getattr(globals()[args.provider], 'get_resources')(args.api_token, args.zone)
     else:
-        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(api_token)
-    provider_info[args.provider]['regions'] =  getattr(globals()[args.provider], 'get_regions')(api_token)
+        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(args.api_token)
+        provider_info[args.provider]['resources'] =  getattr(globals()[args.provider], 'get_resources')(args.api_token)
+
 
     if args.image not in provider_info[args.provider]['images']:
         console.print(f"[red bold]{args.image} is not a supported image")
@@ -150,9 +156,15 @@ def get_args() -> (str, str, argparse.Namespace):
         image_region_choices(args.provider)
         parser.exit(0)
 
-    valid_image = getattr(globals()[args.provider], 'valid_image_in_region')(args.image, args.region, api_token)
+    # make sure the image/region combo is allowed 
+    valid_image = getattr(globals()[args.provider], 'valid_image_in_region')(args.image, args.region, args.api_token)
     assert valid_image, f"{args.image} is not available in {args.region}"
 
+    # make sure the CPU RAM resources are allowed in this region
+    valid_resources = getattr(globals()[args.provider], 'valid_resource_in_region')(args.resources, args.region, args.api_token)
+    assert valid_resources, f"{args.resources} is not available in {args.region}"
+
+
     try:
         if args.help:
             image_region_choices(args.provider)
@@ -280,11 +292,16 @@ def main(args: argparse.Namespace = None, override_teardown: bool = False) -> No
             table.add_column("IP Address", style="magenta")
             table.add_column("Region", style="magenta")
             table.add_column("Image", style="magenta")
+            table.add_column("Hardware", style="magenta")
 
             for host in all_hosts[args.provider]:
                 for hostname, values in host.items():
-                    image = values['workspace'].split('_')[-1]
-                    table.add_row(hostname, values['ip'], values['region'], image.replace('+', '/'))
+                    ip = values['ip'] if 'ip' in values else 'UNK'
+                    region = values['region'] if 'region' in values else 'UNK'
+                    image = values['workspace'].split('_')[-1].replace('+', '/') if 'workspace' in values else 'UNK'
+                    resources = values['resources'] if 'resources' in values else 'UNK'
+
+                    table.add_row(hostname, ip, region, image, resources)
             console.print(table)
         return
 
@@ -335,6 +352,7 @@ def main(args: argparse.Namespace = None, override_teardown: bool = False) -> No
             new_hosts[hostname] = {
                     'ip': ip_address, 
                     'region': args.region,
+                    'resources': args.resources,
                     'workspace': tf_workspace_name
             }
             install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)
@@ -345,6 +363,7 @@ def main(args: argparse.Namespace = None, override_teardown: bool = False) -> No
             new_hosts[hostname] = {
                     'ip': ip_address, 
                     'region': args.region,
+                    'resources': args.resources,
                     'workspace': tf_workspace_name
             }
             install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)
@@ -355,6 +374,7 @@ def main(args: argparse.Namespace = None, override_teardown: bool = False) -> No
             new_hosts[hostname] = {
                     'ip': ip_address, 
                     'region': args.region,
+                    'resources': args.resources,
                     'workspace': tf_workspace_name
             }
             install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)
